# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from wtforms import Form, BooleanField, TextField, TextAreaField, validators
from flask import request, session, render_template, redirect, abort, g, \
                  url_for, flash

from . import app, login_required
from ..model import User, Realm
from .. import model


class AchievementForm(Form):
    name = TextField('Name', [validators.Length(min=4, max=25)])
    description = TextAreaField('Description', [validators.Length(min=6)])


@app.route("/realms/<string:realmname>/achievements/<string:achievementname>/")
def show_achievement(realmname, achievementname):
    realm = Realm.get_by_name(realmname)
    achievement = realm.get_achievement_by_name(achievementname)
    usernames = model.get_usernames_for_achievement(realmname, achievementname)

    return render_template("achievement/show.html.jinja",
                           realm=realm,
                           achievement=achievement,
                           usernames=usernames)


@app.route("/realms/<string:realmname>/achievement/", methods=["GET", "POST"])
@login_required
def create_achievement(realmname):
    """ Creates an achievement in a realm
    """
    realm = Realm.get_by_name(realmname)
    if not realm.is_editable_by(g.user):
        abort(403)

    if request.method == "POST":
        form = AchievementForm(request.form)
        if form.validate():
            realm.add_achievement(form.data["name"],
                                  form.data["description"])
            realm.save()
            flash("Achievement successfully saved")
            return redirect(url_for('realm_details', realmname=realmname))
    else:
        form = AchievementForm()

    return render_template('achievement/create.html.jinja', realm=realm,
                                                            form=form)


@app.route("/realms/<string:realmname>/achievements/"
                     "<string:achievementname>/edit/", methods=["GET", "POST"])
@login_required
def edit_achievement(realmname, achievementname):
    """ Creates an achievement in a realm
    """
    realm = Realm.get_by_name(realmname)
    achievement = realm.get_achievement_by_name(achievementname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.method == "POST":
        form = AchievementForm(request.form)
        if form.validate():
            achievement.update(form.data)
            realm.save()
            flash("Achievement successfully saved")
            return redirect(url_for('show_achievement',
                            realmname=realm.name,
                            achievementname=achievement["name"]))
    else:
        form = AchievementForm(name=achievement["name"],
                               description=achievement["description"])

    return render_template('achievement/edit.html.jinja',
                           realm=realm,
                           achievement=achievement,
                           form=form)


@app.route("/realms/<string:realmname>/achievements/"
         "<string:achievementname>/remove/", methods=["GET", "POST", "DELETE"])
def remove_achievement(realmname, achievementname):
    realm = Realm.get_by_name(realmname)
    achievement = realm.get_achievement_by_name(achievementname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.form.get("_method", request.method) == "DELETE":
        realm.remove_achievement(achievementname)
        flash("Achievement successfully deleted")
        return redirect(url_for('realm_details', realmname=realm.name))

    return render_template('achievement/remove.html.jinja',
                           realm=realm,
                           achievement=achievement)


@app.route("/realms/<string:realmname>/users/<string:username>/",
                                                    methods=["POST", "DELETE"])
@login_required
def grant_achievement(realmname, username):
    realm = Realm.get_by_name(realmname)
    achievement = realm.get_achievement_by_name(
                                               request.form["achievementname"])
    user = realm.get_user_by_username(username)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.form.get("_method", request.method) == "DELETE":
        user.revoke_achievement(realm, achievement)
        flash("achievement revoked")
    else:
        user.grant_achievement(realm, achievement)
        flash("achievement granted")

    user.save()
    return redirect(url_for("show_realm_user", realmname=realmname,
                                               username=username))
