# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from wtforms import Form, BooleanField, TextField, TextAreaField, validators
from flask import request, session, render_template, redirect, g, url_for, \
                  flash

from . import app, login_required
from ..model import User, Realm
from .. import model


class RealmForm(Form):
    name = TextField('Name', [validators.Length(min=4, max=25)])
    description = TextAreaField('Description', [validators.Length(min=6)])


@app.route("/")
def index():
    return render_template("index.html.jinja", realms=Realm.get_all())


@app.route("/realms/new/", methods=["GET", "POST"])
@login_required
def create_realm():
    if request.method == 'POST':
        form = RealmForm(request.form)
        if form.validate():
            realm = Realm()
            realm.owner = g.user.openid
            realm.name = request.form["name"]
            realm.description = request.form["description"]
            realm.save()

            flash("Realm successfully saved")
            return redirect(url_for("realm_details",
                                    realmname=request.form["name"]))
    else:
        form = RealmForm()

    return render_template('realm/create.html.jinja', form=form)


@app.route("/realms/<string:realmname>/edit/", methods=["GET", "POST"])
def edit_realm(realmname):
    realm = Realm.get_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.method == "POST":
        form = RealmForm(request.form)
        if form.validate():
            form.populate_obj(realm)
            realm.save()
            flash("Realm successfully saved")
            return redirect(url_for("realm_details", realmname=realm.name))
    else:
        form = RealmForm(obj=realm)

    return render_template('realm/edit.html.jinja', realm=realm, form=form)


@app.route("/realms/<string:realmname>/remove/",
                                             methods=["GET", "POST", "DELETE"])
def remove_realm(realmname):
    realm = Realm.get_by_name(realmname)

    if not realm.is_editable_by(g.user):
        abort(403)

    if request.form.get("_method", request.method) == "DELETE":
        realm.remove()
        flash("Realm successfully deleted")
        return redirect('/')

    return render_template('realm/remove.html.jinja', realm=realm)


@app.route("/realms/<string:realmname>/")
def realm_details(realmname):
    realm = Realm.get_by_name(realmname)
    owner = realm.get_owner()

    return render_template('realm/show.html.jinja',
                            realm=realm,
                            owner=owner,
                            editable=realm.is_editable_by(g.user),
                            is_member=g.user and g.user.is_realm_member(realm),
                            usernames=realm.get_usernames())


@app.route("/profile/enter_realm/", methods=["POST"])
@login_required
def enter_realm():
    model.add_realm_to_user(g.user,
                            request.form['realmname'],
                            g.user.nick)

    flash("You joined %s. Good luck!" % request.form["realmname"])
    return redirect(url_for("show_profile"))


@app.route("/realms/<string:realmname>/users/<string:username>/")
def show_realm_user(realmname, username):
    realm = Realm.get_by_name(realmname)
    user = realm.get_user_by_username(username)
    membership = user.realms[realmname]
    achievements = realm.doc["achievements"]
    grantable = realm.is_editable_by(g.user)

    return render_template('realm_user.html.jinja', realm=realm,
                                                    membership=membership,
                                                    grantable=grantable,
                                                    achievements=achievements)
