# coding: utf-8
"""
 :copyright: (c) 2011 Philipp Benjamin Köppchen
 :license: GPLv3, see LICENSE for more details.
"""
from __future__ import absolute_import

from flask import g
import pymongo
import bson

from .application import app


@app.before_request
def before_request():
    g.db = pymongo.Connection().scrapmetal


def docproperty(name):
    def get_value(self):
        return self.doc[name]

    def set_value(self, value):
        self.doc[name] = value
    return property(get_value, set_value)


class Realm(object):
    """
    Example document:

    {
        'owner': 'someone.example.com',
        'name': 'Scrapmetal',
        'description': 'This is the Realm dedicated to Scrapmetal itself',
        'achievements': [
            {
                'id': ObjectId('COFFEEBABE'),
                'name': 'Basic Achievement',
                'description': 'This is a basic Achievement, given to everyone'
            },
            # ...
        ]
    }
    """
    name = docproperty("name")
    description = docproperty("description")
    owner = docproperty("owner")

    achievements = docproperty("achievements")  # TODO a little more wrapping?

    def __init__(self, doc=None):
        if doc is None:
            self.doc = {
                'owner': '',
                'name': '',
                'description': '',
                'achievements': [],
            }
        else:
            self.doc = doc

    def is_editable_by(self, user):
        return user is not None and self.owner == user.openid

    def add_achievement(self, name, description):
        self.doc["achievements"].append({
            'id': bson.ObjectId(),
            'name': name,
            'description': description,
        })
        self.save()

    def remove_achievement(self, name):
        self.doc["achievements"] = [a for a in self.doc["achievements"]
                                        if a["name"] != name]
        self.save()

    def get_owner(self):
        return User.get_by_openid(self.owner)

    def get_achievement_by_name(self, achievementname):
        return [achievement for achievement in self.doc["achievements"]
                            if achievement["name"] == achievementname][0]

    def get_granted_achievements(self, user):
        granted_ids = user.doc["realms"][self.name]["achievements"]
        return [achievement for achievement in self.doc["achievements"]
                            if achievement["id"] in granted_ids]

    def get_completion(self, user):
        granted = len(self.get_granted_achievements(user))
        total = len(self.achievements)
        return granted, total, (1.0 * granted / total if total > 0 else 0)

    def save(self):
        g.db.realms.save(self.doc)

    def remove(self):
        g.db.realms.remove(self.doc)

    def get_user_by_username(self, username):
        doc = g.db.users.find_one({'realms.%s.name' % self.name: username})
        return User(doc)

    def get_usernames(self):
        return [user["realms"][self.name]["name"] for user in
                 g.db.users.find({'realms.%s' % self.name: {'$exists': True}})]

    @classmethod
    def get_by_name(cls, name):
        return cls(g.db.realms.find_one({"name": name}))

    @classmethod
    def get_all(cls):
        return [cls(doc) for doc in g.db.realms.find()]


class User(object):
    """
    Example document:

    {
        'openid': 'someone.example.com',
        'nick': 'someone',
        'realms': {
            'Scrapmetal': {
                'name': 'keppla',
                'achievements': ["Basic Achievement", ...]
            }
        }
    }
    """
    nick = docproperty("nick")
    openid = docproperty("openid")
    realms = docproperty("realms")

    def __init__(self, doc=None):
        if doc is None:
            self.doc = {
                'openid': '',
                'realms': {},
            }
        else:
            self.doc = doc

    def save(self):
        g.db.users.save(self.doc)

    @classmethod
    def get_by_openid(cls, openid):
        doc = g.db.users.find_one({"openid": openid})
        return doc and cls(doc)

    @classmethod
    def get_by_nick(cls, nick):
        doc = g.db.users.find_one({"nick": nick})
        return doc and cls(doc)

    def grant_achievement(self, realm, achievement):
        self.realms[realm.name]["achievements"].append(achievement["id"])

    def revoke_achievement(self, realm, achievement):
        self.realms[realm.name]["achievements"].remove(achievement["id"])

    def has_achievement(self, achievement):
        return any(achievement["id"] in membership['achievements']
                                        for membership in self.realms.values())

    def is_realm_member(self, realm):
        return realm.name in self.realms

    def get_realms(self):
        return [Realm(r) for r
                    in g.db.realms.find({"name": {"$in": self.realms.keys()}})]


def add_realm_to_user(user, realmname, username):
    if realmname in user.realms:
        raise RuntimeError("already in realm")

    user.realms[realmname] = {
        'name': username,
        'achievements': [],
    }
    g.db.users.save(user.doc)


def get_usernames_for_achievement(realmname, achievementname):
    users = g.db.users.find({
                        'realms.%s.achievements' % realmname: achievementname})

    return [user['realms'][realmname]['name'] for user in users]
